import traceback
from marshmallow import ValidationError
from sqlalchemy.exc import IntegrityError, OperationalError, ProgrammingError

from omni.pro.logger import LoggerTraceback


def handle_error(self, logger, error, message_response):
    """
    Handle and log errors, returning appropriate response messages.
    Maneja y registra errores, devolviendo mensajes de respuesta apropiados.

    Args:
    logger (Logger object): Logger instance to record the errors.
    Instancia de Logger para registrar los errores.

    error (Exception): The error or exception instance to be handled.
    La instancia de error o excepción a manejar.

    message_response (Response object): Object for creating responses.
    Objeto para crear respuestas.

    Returns:
    Response object: An appropriate error message response.
    Un objeto de respuesta con un mensaje de error apropiado.
    """

    tb = traceback.format_exc()
    logger.error(f"Error: {str(error)}\n\nTraceback:\n\n {tb}")

    error_handlers = {
        IntegrityError: (
            "Location create integrity error",
            lambda error: f"pgerror: {error.orig.pgerror}",
        ),
        ValidationError: (
            "Location create validation error",
            lambda error: str(error.messages),
        ),
        OperationalError: (
            "Location create operational error",
            lambda _: "Error with connection",
        ),
        Exception: (
            "Location create exception",
            lambda _: "Location create Exception",
        ),
        AttributeError: (
            "Location attribute error",
            lambda error: str(error),
        ),
        ProgrammingError: (
            "Location programming error",
            lambda error: str(error),
        ),
        NotFoundError: (
            "Model not found error",
            lambda error: str(error),
        ),
    }

    error_message, error_response = error_handlers.get(type(error), ("Unknown error", None))

    LoggerTraceback.error(error_message, error, logger)

    if isinstance(error, OperationalError):
        return message_response.internal_response(message=error_response)

    return message_response.bad_response(message=error_response)


class NotFoundError(Exception):
    def __init__(
            self,
            message: str | list | dict,
            **kwargs,
    ):
        self.messages = [message] if isinstance(message, (str, bytes)) else message
        self.kwargs = kwargs
        super().__init__(message)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: v1/stock/picking.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from omni.pro.protos.common import base_pb2 as common_dot_base__pb2

DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n\x16v1/stock/picking.proto\x12!pro.omni.oms.api.v1.stock.picking\x1a\x11\x63ommon/base.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1cgoogle/protobuf/struct.proto"\xab\x05\n\x07Picking\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x17\n\x0fpicking_type_id\x18\x03 \x01(\x05\x12\x13\n\x0blocation_id\x18\x04 \x01(\x05\x12\x18\n\x10location_dest_id\x18\x05 \x01(\x05\x12\x0f\n\x07user_id\x18\x06 \x01(\x05\x12\x1b\n\x13\x61ttachment_guide_id\x18\x07 \x01(\x05\x12\x1d\n\x15\x61ttachment_invoice_id\x18\x08 \x01(\x05\x12\x0e\n\x06origin\x18\t \x01(\t\x12:\n\x16\x64\x61te_start_preparation\x18\n \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\tdate_done\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x32\n\x0escheduled_date\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x1e\n\x16time_total_preparation\x18\r \x01(\x02\x12\x15\n\rtime_assigned\x18\x0e \x01(\x02\x12\x12\n\ncarrier_id\x18\x0f \x01(\x05\x12\x31\n\rdate_delivery\x18\x10 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x1c\n\x14\x63\x61rrier_tracking_ref\x18\x11 \x01(\t\x12\x10\n\x08group_id\x18\x12 \x01(\x05\x12\x0e\n\x06weight\x18\x13 \x01(\x02\x12\x17\n\x0fshipping_weight\x18\x14 \x01(\x02\x12*\n\x06\x61\x63tive\x18\x15 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12?\n\x0cobject_audit\x18\x16 \x01(\x0b\x32).pro.omni.oms.api.common.base.ObjectAudit"\xf7\x04\n\x14PickingCreateRequest\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x17\n\x0fpicking_type_id\x18\x02 \x01(\x05\x12\x13\n\x0blocation_id\x18\x03 \x01(\x05\x12\x18\n\x10location_dest_id\x18\x04 \x01(\x05\x12\x0f\n\x07user_id\x18\x05 \x01(\x05\x12\x1b\n\x13\x61ttachment_guide_id\x18\x06 \x01(\x05\x12\x1d\n\x15\x61ttachment_invoice_id\x18\x07 \x01(\x05\x12\x0e\n\x06origin\x18\x08 \x01(\t\x12:\n\x16\x64\x61te_start_preparation\x18\t \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\tdate_done\x18\n \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x32\n\x0escheduled_date\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x1e\n\x16time_total_preparation\x18\x0c \x01(\x02\x12\x15\n\rtime_assigned\x18\r \x01(\x02\x12\x12\n\ncarrier_id\x18\x0e \x01(\x05\x12\x31\n\rdate_delivery\x18\x0f \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x1c\n\x14\x63\x61rrier_tracking_ref\x18\x10 \x01(\t\x12\x10\n\x08group_id\x18\x11 \x01(\x05\x12\x0e\n\x06weight\x18\x12 \x01(\x02\x12\x17\n\x0fshipping_weight\x18\x13 \x01(\x02\x12\x36\n\x07\x63ontext\x18\x14 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x9f\x01\n\x15PickingCreateResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12;\n\x07picking\x18\x02 \x01(\x0b\x32*.pro.omni.oms.api.v1.stock.picking.Picking"\xf0\x02\n\x12PickingReadRequest\x12\x37\n\x08group_by\x18\x01 \x03(\x0b\x32%.pro.omni.oms.api.common.base.GroupBy\x12\x35\n\x07sort_by\x18\x02 \x01(\x0b\x32$.pro.omni.oms.api.common.base.SortBy\x12\x34\n\x06\x66ields\x18\x03 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Fields\x12\x34\n\x06\x66ilter\x18\x04 \x01(\x0b\x32$.pro.omni.oms.api.common.base.Filter\x12:\n\tpaginated\x18\x05 \x01(\x0b\x32\'.pro.omni.oms.api.common.base.Paginated\x12\n\n\x02id\x18\x06 \x01(\x05\x12\x36\n\x07\x63ontext\x18\x07 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\xd9\x01\n\x13PickingReadResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12\x39\n\tmeta_data\x18\x02 \x01(\x0b\x32&.pro.omni.oms.api.common.base.MetaData\x12<\n\x08pickings\x18\x03 \x03(\x0b\x32*.pro.omni.oms.api.v1.stock.picking.Picking"\x8b\x01\n\x14PickingUpdateRequest\x12;\n\x07picking\x18\x01 \x01(\x0b\x32*.pro.omni.oms.api.v1.stock.picking.Picking\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x9f\x01\n\x15PickingUpdateResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12;\n\x07picking\x18\x02 \x01(\x0b\x32*.pro.omni.oms.api.v1.stock.picking.Picking"Z\n\x14PickingDeleteRequest\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"b\n\x15PickingDeleteResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard"\\\n\x16ValidatePickingRequest\x12\n\n\x02id\x18\x01 \x01(\x05\x12\x36\n\x07\x63ontext\x18\x02 \x01(\x0b\x32%.pro.omni.oms.api.common.base.Context"\x8b\x01\n\x17ValidatePickingResponse\x12I\n\x11response_standard\x18\x01 \x01(\x0b\x32..pro.omni.oms.api.common.base.ResponseStandard\x12%\n\x04\x64\x61ta\x18\x02 \x01(\x0b\x32\x17.google.protobuf.Struct2\xb2\x05\n\x0ePickingService\x12\x84\x01\n\rPickingCreate\x12\x37.pro.omni.oms.api.v1.stock.picking.PickingCreateRequest\x1a\x38.pro.omni.oms.api.v1.stock.picking.PickingCreateResponse"\x00\x12~\n\x0bPickingRead\x12\x35.pro.omni.oms.api.v1.stock.picking.PickingReadRequest\x1a\x36.pro.omni.oms.api.v1.stock.picking.PickingReadResponse"\x00\x12\x84\x01\n\rPickingUpdate\x12\x37.pro.omni.oms.api.v1.stock.picking.PickingUpdateRequest\x1a\x38.pro.omni.oms.api.v1.stock.picking.PickingUpdateResponse"\x00\x12\x84\x01\n\rPickingDelete\x12\x37.pro.omni.oms.api.v1.stock.picking.PickingDeleteRequest\x1a\x38.pro.omni.oms.api.v1.stock.picking.PickingDeleteResponse"\x00\x12\x8a\x01\n\x0fValidatePicking\x12\x39.pro.omni.oms.api.v1.stock.picking.ValidatePickingRequest\x1a:.pro.omni.oms.api.v1.stock.picking.ValidatePickingResponse"\x00\x62\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, "v1.stock.picking_pb2", _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
    DESCRIPTOR._options = None
    _globals["_PICKING"]._serialized_start = 176
    _globals["_PICKING"]._serialized_end = 859
    _globals["_PICKINGCREATEREQUEST"]._serialized_start = 862
    _globals["_PICKINGCREATEREQUEST"]._serialized_end = 1493
    _globals["_PICKINGCREATERESPONSE"]._serialized_start = 1496
    _globals["_PICKINGCREATERESPONSE"]._serialized_end = 1655
    _globals["_PICKINGREADREQUEST"]._serialized_start = 1658
    _globals["_PICKINGREADREQUEST"]._serialized_end = 2026
    _globals["_PICKINGREADRESPONSE"]._serialized_start = 2029
    _globals["_PICKINGREADRESPONSE"]._serialized_end = 2246
    _globals["_PICKINGUPDATEREQUEST"]._serialized_start = 2249
    _globals["_PICKINGUPDATEREQUEST"]._serialized_end = 2388
    _globals["_PICKINGUPDATERESPONSE"]._serialized_start = 2391
    _globals["_PICKINGUPDATERESPONSE"]._serialized_end = 2550
    _globals["_PICKINGDELETEREQUEST"]._serialized_start = 2552
    _globals["_PICKINGDELETEREQUEST"]._serialized_end = 2642
    _globals["_PICKINGDELETERESPONSE"]._serialized_start = 2644
    _globals["_PICKINGDELETERESPONSE"]._serialized_end = 2742
    _globals["_VALIDATEPICKINGREQUEST"]._serialized_start = 2744
    _globals["_VALIDATEPICKINGREQUEST"]._serialized_end = 2836
    _globals["_VALIDATEPICKINGRESPONSE"]._serialized_start = 2839
    _globals["_VALIDATEPICKINGRESPONSE"]._serialized_end = 2978
    _globals["_PICKINGSERVICE"]._serialized_start = 2981
    _globals["_PICKINGSERVICE"]._serialized_end = 3671
# @@protoc_insertion_point(module_scope)
